#!/usr/local/bin/perl -w

use strict;
use Tk;
use Tk::Dialog;
use Tk::DialogBox;
use Tk::LabEntry;
use Net::Ping; # zur Kontrolle, ob Rechner online ist


sub create_scrolled_list {
    my ($frame) = @_;
    my $xsb = $frame->Scrollbar(-orient => "horizontal");
			  
    my $ysb = $frame->Scrollbar(-orient => "vertical");
			   
    my $list = $frame->Listbox(-selectmode => "single",
			       -yscrollcommand => ['set', $ysb] ,
			       -xscrollcommand => ['set' , $xsb] );
    $xsb->configure( -command => [ 'xview', $list ]);
    $ysb->configure( -command => [ 'yview' , $list ]);
    $xsb->pack(-side => "bottom", -fill => "x");
    $ysb->pack(-side => "right", -fill => "y");
    $list->pack(-expand => 1, -fill=> "both");
    return $list;
}

#
# gibt's gtar?. Entweder als "gtar" oder "tar" versteht "--help".
# Sonst -> "dummes" tar
#
my $tar_name = "gtar";
`which $tar_name > /dev/null`;
my $gtar = ! $?;
if (! $gtar) {
    $tar_name = "tar";
    my $found = `which $tar_name`;
    chop $found;
    `$found --help`;
    $gtar = ! $?;
}

my $main = MainWindow->new;
my ($xsb, $ysb);
my (@hostnames, $name,$hostcount, $ignore, $Dialog1, $Dialog2, $Dialog3,@address);
my ($deviceList, $directoryList);
my ($a1, $a2, $a3, $a4);

# Erster Frame fr Gerte- und Rechner-Wahl
my $f1 = $main->Frame()->pack(-side => "top");
# Zweiter Frame fr Verzeichnisse & Optionen
my $f2 = $main->Frame()->pack(-side => "top",-anchor => "w");
my $frame = $f1->Frame()->pack(-side => "left", -fill => "both");


#
# Rechner finden, die von hier aus erreichbar sind.
# Nicht perfekt: liest /etc/hosts
#
my $list = create_scrolled_list($frame);
sethostent(1);

$hostcount = 0;

while (($name, $ignore, $ignore, $ignore, @address) = gethostent) {
    ($a1, $a2, $a3, $a4) = unpack('C4', $address[0]);
    $list->insert("end", 
		  $name . " (" . $a1 . "." . $a2 . "." . $a3 . "." . $a4 . ")" );
    $list->selection("set", "end") if ($name eq "localhost");
    $hostnames[$hostcount] = $name;
    $hostcount++;
}
endhostent();


$list->bind("<ButtonRelease-1>" => [ \&check_host ]);
#
# Nur gtar kann auf remote angeschlossene Gerte schreiben
#
$list->configure("state", "disabled") if (! $gtar);


my $device = (get_device_list("localhost"))[0];
my $user = (getpwuid($<))[0];
my $directory = "/";
my $samefs = "";
my $incremental = 0;
my $date;
$frame= $f1->Frame()->pack(-side => "left");

$frame->LabEntry(-label => "Benutzer: ",
		 -textvariable => \$user,
		 -pack => "left",
		 )->pack(-side => "top");
$frame->LabEntry(-label => "Gert: ",  
		-textvariable => \$device
		)->pack(-side=>"top");
$frame->Button(-text => "Gert suchen...",
	      -command => [ \&find_devices, \$device ]
	      )->pack(-side => "top");

$frame= $f2->Frame()->pack(-side => "left", -anchor => "w");

$frame->LabEntry(-label => "Verzeichnisse: ",
		 -textvariable => \$directory
		 )->pack(-side=>"top", -anchor => "w");
$frame->Button(-text => "Verzeichnisse whlen...",
	      -command => [ \&find_directory ]
	      )->pack(-side => "top", -anchor => "w");
$frame->Checkbutton(-text => "Im Filesystem bleiben",
		    -variable => \$samefs,
		    -offvalue => "",
		    -onvalue => "-xdev"
		    )->pack(-side => "top", -anchor => "w");
$frame= $f2->Frame()->pack(-side => "left", -fill => "y", -expand => 1);
$frame->Radiobutton(-text => "Volles Backup",
		    -variable => \$incremental,
		    -value => 0,
		    -command => [ \&date_state ]
		    )->pack(-side => "top", -anchor => "w");
$frame->Radiobutton(-text => "Inkrementelles Backup",
		    -variable => \$incremental,
		    -value => 1,
		    -command => [ \&date_state ],
		    -state => $gtar ? "normal" : "disabled"
		    )->pack(-side => "top", -anchor => "w");
my $date_field = $frame->LabEntry(-label => "Dateien jnger als (tt/mm/jj): ",
				  -textvariable => \$date,
				  -state => "disabled",
			       )->pack(-side=>"top", -anchor => "w");
$date_field->Subwidget('label')->configure(-foreground => "gray");
$date_field->Subwidget('entry')->bind("<KeyPress>" => [ \&check_date ]);
$frame = $main->Frame()->pack(-side => "top", 
			      -expand => 1);
$frame->Button(-text => "Schreiben", 
	       -command => [ \&write_tar ] 
	       )->pack(-side => "left", -expand => 1 , -fill => "x");

$frame->Button(-text => "Ende", 
	       -command => [ \&exit ] 
	       )->pack(-side => "left", -expand => 1 , -fill => "x");

	       
				      
MainLoop;

sub write_tar {
    my $command;
    if ($gtar) {
	$command = "$tar_name -c";
	$command = $command . "l" if ($samefs ne "");
	$command = $command . "f $device ";
	$command = $command . "-N". convert_date()  if ($incremental);
	$command = $command . " " . $directory;
	printf "%s\n", $command;
    }
}


sub warn_host {
    my ($ind,$host)  = @_;

    my $msg = "Rechner '" . $host . "' antwortet nicht.";
    my($ign, $del) =  ('Ignorieren', 'Rechner entfernen');
    if (not Exists($Dialog1)) {
	$Dialog1 = $main->Dialog(
	    -title          => 'Keine Verbindung zum Rechner!',
            -text           => '',
            -bitmap         => 'warning',
            -default_button => $ign,
            -buttons        => [$ign, $del],
        );
	$Dialog1->configure(
            -wraplength => '4i',
          -text      => $msg,
        );
    }
    my $button = $Dialog1->Show();
    if ($button eq $del) {
	$list->delete($ind);
	$list->selection("set",0);
    }
}

#
# berprft, ob der angewhlte Rechner erreichbar ist
# Erstes Argument ist implizit die Liste
#
sub check_host {
    my ($l) = @_;
    my $i = $l->curselection();
    if (undef $i) {
	$l->selection("set", 0);
	return;
    }
    my ($select) = $hostnames[$l->curselection()];
    my $p      =  Net::Ping->new();
    my $result = $p->ping($select);
    undef $p;  # $p->close() gibt Fehlermeldung!
    warn_host($l->curselection(), $l->get($l->curselection())) if (! $result);
}


sub get_device_list {
    my ($host) = @_;
    my ($list, @devices, @result, $i, $d);

#
#  Einige Gerte, die fr Backups nicht taugen
#
    my $exclude = "^([pt]ty|win|zsh|zero|null|swap|audio|par|midi|sys|console).*\$";
    if ($host ne "localhost") {
	if ($user ne "") {
	    $list = `rsh -l $user $host ls -l /dev `;
	} else {
	    $list = `rsh $host ls -l /dev`;
	}
    } else {
	$list = `ls -l /dev`;
    }
    @devices = split(/^/,$list);
    foreach $i (@devices) {
	chop $i;
	if ($i =~ /^[cb].w..w..w./) {
	    $i =~ s/.* (\S+)$/$1/;
	    if ( $i !~ /$exclude/ ) {
		push @result, "/dev/" . $i;
	    }
	}
    }
    return @result;
}
    
#
# Sucht auf "host" nach Gerten in /dev, die echte Character-
# oder Block-Devices sind und auf die _alle_ schreiben knnen.
# Problem: Unter Solaris liegen in /dev nur symbolische Links
# nach ../devices, so da in diesem Fall
# keine Gerte gefunden werden.
#
sub find_devices {
    my $host = $hostnames[$list->curselection()];
    my ($d, @devices,$i);
    my($ok, $can) =  ('OK', 'Abbrechen');
    my ($frame,$name);

    if (not Exists($Dialog2)) {
	$Dialog2 = $main->DialogBox(
	    -title          => 'Gert auswhlen',
            -text           => 'Gert fr Backup auswhlen',
            -default_button => $ok,
            -buttons        => [$ok, $can]
        );
	$frame = $Dialog2->add('Frame');
	$frame->pack(-side => "left", -fill => "both");
	$deviceList = create_scrolled_list($frame);
	
    }
    $deviceList->delete(0,"end");
    $deviceList->insert("end", get_device_list($host));
    $deviceList->selection("set", 0);
    my $button = $Dialog2->Show();
    if ($button eq $ok) {
	$device = $deviceList->get($deviceList->curselection());
    }
}

sub find_directory {
    my ($i, $start);
    my ($frame);
    my($ok, $down, $up, $can) =  ('OK', "ffnen", "Zurck", 'Abbrechen');

    if (not Exists($Dialog3)) {
	$Dialog3 = $main->Toplevel(
	    -title          => 'Verzeichnis');
	$frame = $Dialog3->Frame()->pack(-side => "top");
	$directoryList = create_scrolled_list($frame);
	$frame = $Dialog3->Frame()->pack(-side => "top");
	$frame->Button(-text => $ok,
		       -command => [ \&set_directory ]
		       )->pack(-side => "left");
	$frame->Button(-text => $down,
		       -command => [ \&update_list, "."]
		       )->pack(-side => "left");
	$frame->Button(-text => $up,
		       -command => [ \&update_list, ".." ]
		       )->pack(-side => "left");
	$frame->Button(-text => $can,
		       -command => [ sub  {
				    $Dialog3->UnmapWindow() 
				    } ]
		       )->pack(-side => "left");
    } 
    $directoryList->selection("set",0);
    update_list($directory);
    $Dialog3->Popup();    
}

sub set_directory {
    $directory = $directoryList->get($directoryList->curselection());
    $Dialog3->UnmapWindow();
}

sub update_list {
    my ($first) = @_;
    my $i = $directoryList->curselection();
    my $curdir = `pwd`;
    $i = 0 if (!defined $i);
    my $sel = $directoryList->get($i);
    $sel = "/" if (!defined $sel);
    if ($first eq "") {
	$first = $sel;
    }
    $directoryList->delete(0, "end");
    if ($first eq "/") {
	$directoryList->insert("end","/");
    } else {
	if ($first eq ".") {
	    $first=$sel;
	} elsif ($first eq "..") {
	    $curdir = `pwd`;
	    chdir $sel . "/../..";
	    $first = `pwd`;
	    chop $first;
	    chdir $curdir;
	    if ($first eq "/") {
		$directoryList->insert("end","/");
		return;
	    }
	}
	# Alle Unterdirectories lesen
	my $found = `find $first -maxdepth 1 $samefs -type d -print`;
	my @dlist = split(/^/, $found);
	foreach $i (@dlist) {
	    chop $i;
	    $directoryList->insert("end",$i) if ($i ne $first) ;
	}
    }
}

sub date_state {
    $date_field->configure(-state => $incremental ? "enabled" : "disabled");
    $date_field->Subwidget('label')->configure(-foreground => 
					       $incremental ? 
					       "black" : "gray");
}

sub convert_date {
    my ($t,$m,$y) = split(/\//,$date);
    my $tardate = $m . "/" . $t . "/" . $y;
    return $tardate;
}

#
# Rudimentre Datumsprfung. Nur "/" und [0-9] sind zulssig
# Ob die eingegebenen Zeichen berhaupt ein sinnvolles Datum
# bilden, wird nicht berprft.
#
sub check_date {
    my ($w) = @_;
    my $k = $w->XEvent->A;
    return if (! defined $k || length $k == 0);
    my $entry = $date_field->Subwidget('entry');
    my $insert = $entry->index("insert")-1;
    return if ($insert < 0) ;
    $entry->delete($insert)  if ($k !~ /[0-9\/]/) ;

}

